@@ -18,12 +18,31 @@ The access backend needs to be configured with an access token that has read-acc
 import abc
 import logging
 import re
-from enum import Enum
-from typing import Any, Dict, List, Optional, Set, Tuple, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Literal,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    TypedDict,
+    TypeVar,
+    Union,
+    cast,
+    overload,
+)
 
-from outcome.pypicloud_access_github.graphql import schema
-from outcome.pypicloud_access_github.graphql.client import Client, Operation
+from dogpile.cache import CacheRegion
+from outcome.pypicloud_access_github.graphql.github import Github
+from outcome.pypicloud_access_github.graphql.schemas.models.organization import OrganizationMemberRole
+from outcome.pypicloud_access_github.graphql.schemas.models.repository import DefaultRepositoryPermissionField
+from outcome.pypicloud_access_github.graphql.schemas.models.team import TeamMemberRole
 from outcome.utils import cache
+from passlib.apps import LazyCryptContext
 from pypicloud.access.base import ONE_WEEK, IAccessBackend
 from pyramid.settings import aslist
 
@@ -31,100 +50,117 @@ LOG = logging.getLogger(__name__)
 
 # Build a new dogpile.cache cache region
 # we'll configure it later when the Access.configure method is called
-cache_region = cache.get_cache_region()
+cache_region: CacheRegion = cache.get_cache_region()
 
 
 _read_permission = 'read'
 _write_permission = 'write'
 
 
-class GithubRoles(Enum):
-    admin = 'admin'
-    maintain = 'maintain'
-    write = 'write'
-    triage = 'triage'
-    read = 'read'
+PypiPermission = Union[Literal['read'], Literal['write']]
 
 
 # Mapping from Github roles to pypi permissions
-_permissions_map = {
-    GithubRoles.admin.value: {_read_permission, _write_permission},
-    GithubRoles.maintain.value: {_read_permission, _write_permission},
-    GithubRoles.write.value: {_read_permission, _write_permission},
-    GithubRoles.triage.value: {_read_permission},
-    GithubRoles.read.value: {_read_permission},
+_permissions_map: Dict[DefaultRepositoryPermissionField, Set[PypiPermission]] = {
+    DefaultRepositoryPermissionField.admin: {_read_permission, _write_permission},
+    DefaultRepositoryPermissionField.write: {_read_permission, _write_permission},
+    DefaultRepositoryPermissionField.read: {_read_permission},
+    DefaultRepositoryPermissionField.none: set(),
 }
 
 _teams_key = 'teams'
 _users_key = 'users'
 
+Entity = Literal['users', 'teams']
 
-UserRole = Tuple[str, str]
-TeamMemberMap = Dict[str, List[UserRole]]
-User = Dict[str, Union[str, bool, List[str]]]
-UserWithGroups = User
-EntityPermissions = Dict[str, Set[str]]
-RepositoryPermissions = Dict[str, EntityPermissions]
+UserRole = Tuple[str, TeamMemberRole]
+TeamMemberMap = Dict[str, Sequence[UserRole]]
+
+C = TypeVar('C', bound=Callable[..., Any])
+
+
+# A workaround the incorrect typing on the cache_on_arguments method
+def cacheable(fn: C) -> C:
+    cache_region.cache_on_arguments()(fn)
+    return fn
+
+
+class User(TypedDict):
+    username: str
+    admin: bool
+    groups: Optional[Sequence[str]]
+
+
+class RepositoryPermissions(TypedDict):
+    users: Dict[str, Set[PypiPermission]]
+    teams: Dict[str, Set[PypiPermission]]
+
+
+class PackagePermissions(TypedDict):
+    package: str
+    permissions: Sequence[PypiPermission]
 
 
 class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-tests; # noqa: WPS214
+
+    client: Github
+    organization_name: str
+    repo_exclude_list: Optional[List[str]]
+    repo_include_list: Optional[List[str]]
+
     def __init__(  # noqa: WPS211, too many arguments
         self,
-        request=None,
-        default_read=None,
-        default_write=None,
-        disallow_fallback=(),
-        cache_update=None,
-        pwd_context=None,
-        token_expiration=ONE_WEEK,
-        signing_key=None,
-        token: str = None,
-        organization: str = None,
+        request: Optional[object] = None,
+        default_read: Optional[Sequence[str]] = None,
+        default_write: Optional[Sequence[str]] = None,
+        disallow_fallback: Optional[Iterable[str]] = (),
+        cache_update: Optional[Sequence[str]] = None,
+        pwd_context: Optional[LazyCryptContext] = None,
+        token_expiration: int = ONE_WEEK,
+        signing_key: Optional[str] = None,
+        token: Optional[str] = None,
+        organization: Optional[str] = None,
         repo_pattern: Optional[str] = None,
-        repo_include_list: Optional[str] = None,
-        repo_exclude_list: Optional[str] = None,
+        repo_include_list: Optional[List[str]] = None,
+        repo_exclude_list: Optional[List[str]] = None,
     ) -> None:
 
         super().__init__(
             request, default_read, default_write, disallow_fallback, cache_update, pwd_context, token_expiration, signing_key,
         )
 
-        self.organization_name = organization
+        assert organization
+        assert token
 
-        self.client = Client(token)
+        self.organization_name = organization
+        self.client = Github(token)
 
         self.repo_pattern = repo_pattern
         self.repo_include_list = repo_include_list
         self.repo_exclude_list = repo_exclude_list
 
-    def organization_operation(self) -> Tuple[Operation, schema.Organization]:
-        op = self.client.operation()
-        org = op.organization(login=self.organization_name)
-
-        return (op, org)
-
     @abc.abstractmethod
-    def get_package_name(self, package_file_content: str) -> Optional[str]:
+    def get_package_name(self, package_file_content: str) -> Optional[str]:  # pragma: no cover
         ...
 
     @property
     @abc.abstractmethod
-    def package_file_name(self) -> str:
+    def package_file_name(self) -> str:  # pragma: no cover
         ...
 
     @staticmethod
-    def convert_permission(github_role: str) -> Set[str]:
-        """Convert a Github role ('admin', 'triage', etc. ) to read/write permissions.
+    def convert_permission(github_role: DefaultRepositoryPermissionField) -> Set[PypiPermission]:
+        """Convert a Github role to read/write permissions.
 
         Args:
-            github_role (str): The Github role.
+            github_role (DefaultRepositoryPermissionField): The Github role.
 
         Returns:
             Set[str]: The set of permissions (either 'read' or 'write', or both)
         """
-        return _permissions_map[github_role.lower()]
+        return _permissions_map[github_role]
 
-    @cache_region.cache_on_arguments()
+    @cacheable
     def package_files(self) -> Dict[str, Optional[str]]:
         """Retrieve the map of repositories and the contents of the package files.
 
@@ -139,23 +175,11 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         LOG.info('Getting package files')
 
-        op, org = self.organization_operation()
+        package_files = self.client.get_organization_package_files(self.organization_name, self.package_file_name)
 
-        # Get the name of all of the repositories
-        repository = org.repositories.edges.node
-        repository.name()
+        return {repo.name: content for repo, content in package_files}
 
-        # Get the object from the `master` branch
-        package_file = repository.object(expression=f'master:{self.package_file_name}')
-
-        # Get the text content of the package file
-        package_file.__as__(schema.Blob).text()
-
-        data = self.client.get_all(op, page_on='organization.repositories')
-
-        return {r.node.name: getattr(r.node.object, 'text', None) for r in data.organization.repositories.edges}
-
-    @cache_region.cache_on_arguments()
+    @cacheable
     def package_names(self) -> Dict[str, str]:  # noqa: WPS231, complexity
         """Returns the list of available packages from the GitHub Organization.
 
@@ -176,7 +200,7 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         LOG.info('Getting package names')
 
-        packages = {}
+        packages: Dict[str, str] = {}
 
         for repository_name, package_file_content in self.package_files().items():
             if not package_file_content:
@@ -199,10 +223,12 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         return packages
 
     @classmethod
-    def configure(cls, settings) -> Dict[str, Any]:
+    def configure(cls, settings: Dict[str, object]) -> Dict[str, object]:
         base_config = super().configure(settings)
         organization = settings.get('auth.otc.github.organization')
 
+        assert isinstance(organization, str)
+
         LOG.info(f'Configuring Github Auth with organization: {organization}')
 
         # Configure the cache region
@@ -219,7 +245,7 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
             'repo_exclude_list': aslist(settings.get('auth.otc.github.repo_exclude_list', [])),
         }
 
-    @cache_region.cache_on_arguments()
+    @cacheable
     def is_valid_token_for_username(self, username: str, token: str) -> bool:
         """Check that the token is associated with the username.
 
@@ -234,20 +260,17 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         # credentials
         LOG.info('Checking user token')
 
-        user_client = Client(token)
+        try:
+            user_client = Github(token)
+            current_user = user_client.get_current_user()
 
-        op = user_client.operation()
+            # Ensure the username matches the token
+            return current_user.login == username
+        except Exception:
+            LOG.exception('User verification failed')
+            return False
 
-        # Get the user node, and its login
-        user = op.viewer()
-        user.login()
-
-        result = user_client.execute(op)
-
-        # Ensure the username matches the token
-        return hasattr(result, 'viewer') and getattr(result.viewer, 'login', None) == username  # noqa: WPS421, has/getattr
-
-    @cache_region.cache_on_arguments()
+    @cacheable
     def verify_user(self, username: str, password: str) -> bool:
         """Check the login credentials of a user.
 
@@ -266,7 +289,7 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
 
         return username in self.users()
 
-    @cache_region.cache_on_arguments()
+    @cacheable
     def teams(self) -> TeamMemberMap:
         """Returns a map of GitHub Team names and members.
 
@@ -277,31 +300,12 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         LOG.info('Getting teams')
 
-        op, org = self.organization_operation()
+        teams = self.client.get_organization_teams(self.organization_name)
 
-        team = org.teams.edges.node
-        team.name()
+        return {name: [(user.login, role) for user, role in members] for name, members in teams.items()}
 
-        membership = team.members.edges
-        membership.role()
-
-        member = membership.node
-        member.login()
-
-        data = self.client.get_all(op, page_on='organization.teams')
-        teams = {}
-
-        for t in data.organization.teams.edges:
-            members = []
-            teams[t.node.name] = members
-
-            for m in t.node.members.edges:
-                members.append((m.node.login, m.role))
-
-        return teams
-
-    @cache_region.cache_on_arguments()
-    def groups(self, username: Optional[str] = None) -> List[str]:
+    @cacheable
+    def groups(self, username: Optional[str] = None) -> Sequence[str]:  # type: ignore
         """Get a list of all groups.
 
         If a username is specified, get all groups to which the user belongs.
@@ -314,11 +318,11 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         if username:
             user_data = self.user_data(username)
-            return user_data['groups']
+            return user_data['groups'] or []
 
         return list(self.teams().keys())
 
-    @cache_region.cache_on_arguments()
+    @cacheable
     def group_members(self, group: str) -> List[str]:
         """Get a list of users that belong to a group.
 
@@ -328,9 +332,9 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         Returns:
             List[str]: The list usernames of the members of the group.
         """
-        return [username for username, role in self.teams().get(group, [])]
+        return [username for username, _ in self.teams().get(group, [])]
 
-    @cache_region.cache_on_arguments()
+    @cacheable
     def is_admin(self, username: str) -> bool:
         """Check if the user is an admin.
 
@@ -342,7 +346,7 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         return self.users().get(username, False)
 
-    def group_permissions(self, package: str) -> Dict[str, List[str]]:
+    def group_permissions(self, package: str) -> Dict[str, List[PypiPermission]]:
         """Get a mapping of all groups to their permissions on a package.
 
         Args:
@@ -353,7 +357,7 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         return self.package_permissions(package, _teams_key)
 
-    def user_permissions(self, package: str) -> Dict[str, List[str]]:
+    def user_permissions(self, package: str) -> Dict[str, List[PypiPermission]]:
         """Get a mapping of all users to their permissions for a package.
 
         Args:
@@ -364,16 +368,16 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         return self.package_permissions(package, _users_key)
 
-    @cache_region.cache_on_arguments()
-    def package_permissions(self, package: str, principal_type: str) -> Dict[str, List[str]]:
+    @cacheable
+    def package_permissions(self, package: str, principal_type: Entity) -> Dict[str, List[PypiPermission]]:
         """Get a mapping of all entities of the principal type to their permissions for a package.
 
         Args:
             package (str): The name of a python package.
-            principal_type (str): The entity type ('users' or 'teams')
+            principal_type (Entity): The entity type ('users' or 'teams')
 
         Returns:
-            Dict[str, List[str]]: Mapping of username to a list of permissions (which can contain 'read' and/or 'write')
+            Dict[str, List[PypiPermission]]: Mapping of username to a list of permissions
         """
         assert principal_type in {_users_key, _teams_key}  # noqa: S101, use of assert
 
@@ -386,23 +390,23 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
 
         return {u: list(p) for u, p in repo_perms[principal_type].items()}
 
-    @cache_region.cache_on_arguments()
-    def entity_package_permissions(self, entity_type: str, entity_name: str) -> List[Dict[str, Union[List[str], str]]]:
+    @cacheable
+    def entity_package_permissions(self, entity_type: Entity, entity_name: str) -> Sequence[PackagePermissions]:
         """Get a list of all packages that a user has permissions on.
 
         Args:
-            entity_type (str): The name of the entity.
+            entity_type (Entity): The name of the entity.
             entity_name (str): The entity type ('users', or 'teams')
 
         Returns:
-            List[Dict[str, Union[List[str], str]]]: List of dicts. Each dict contains 'package' (str) and 'permissions'.
+            List[PackagePermissions]: List of dicts. Each dict contains 'package' (str) and 'permissions'.
         """
         LOG.info('Getting package permissions')
 
         assert entity_type in {_users_key, _teams_key}  # noqa: S101, use of assert
 
         all_repo_perms = self.repository_permissions()
-        permissions = []
+        permissions: List[PackagePermissions] = []
 
         # For each possible package, we want to retrieve the permissions
         # associated with the package's repo
@@ -414,13 +418,16 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
             if entity_name not in repo_perms[entity_type]:
                 continue
 
+            # NOTE: sets of literals seem to be cast as sequences of str, so we need to recast
+            entity_permissions = cast(Sequence[PypiPermission], list(repo_perms[entity_type][entity_name]))
+
             # Append the permissions
-            package_perms = dict(package=package, permissions=list(repo_perms[entity_type][entity_name]))
+            package_perms: PackagePermissions = {'package': package, 'permissions': entity_permissions}
             permissions.append(package_perms)
 
         return permissions
 
-    def user_package_permissions(self, username: str) -> List[Dict[str, Union[List[str], str]]]:
+    def user_package_permissions(self, username: str) -> Sequence[PackagePermissions]:  # type: ignore
         """Get a list of all packages that a user has permissions on.
 
         Args:
@@ -432,7 +439,7 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         return self.entity_package_permissions(_users_key, username)
 
-    def group_package_permissions(self, group: str) -> List[Dict[str, Union[List[str], str]]]:
+    def group_package_permissions(self, group: str) -> Sequence[PackagePermissions]:  # type: ignore
         """Get a list of all packages that a group has permissions on.
 
         Args:
@@ -444,7 +451,7 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         return self.entity_package_permissions(_teams_key, group)
 
-    @cache_region.cache_on_arguments()
+    @cacheable
     def repository_permissions(self) -> Dict[str, RepositoryPermissions]:  # noqa: WPS231, cognitive complexity
         """Retrieve the permission set for all the repositories.
 
@@ -477,69 +484,43 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         LOG.info('Getting repository permissions')
 
-        repositories = {}
+        repositories: Dict[str, RepositoryPermissions] = {}
 
-        op, org = self.organization_operation()
+        all_repo_permissions = self.client.get_organization_repository_permissions(self.organization_name)
 
-        repository = org.repositories.edges.node
-        repository.name()
+        for repo, user_permissions, team_permissions in all_repo_permissions:
+            repo_perms: RepositoryPermissions = {'users': {}, 'teams': {}}
 
-        collaboration = repository.collaborators.edges
+            for user, u_permissions in user_permissions:
+                user_perms: Set[PypiPermission] = cast(Set[PypiPermission], set())
 
-        # We want the list of users
-        user = collaboration.node
-        user.login()
+                for u_p in u_permissions:
+                    user_perms = user_perms.union(self.convert_permission(u_p))
 
-        # We want to get all of the sources of permissions for the user on the
-        # repo. The user can have permissions directly from the repo, or via a
-        # team on the repo, or via the organization that contains the repo
-        permission_source = collaboration.permission_sources
-        permission_source.permission()
+                repo_perms['users'][user.login] = user_perms
 
-        permission_source.source.__as__(schema.Team).name()
-        permission_source.source.__as__(schema.Repository).name()
-        permission_source.source.__as__(schema.Organization).login()
+            for team, t_permissions in team_permissions:
+                team_perms: Set[PypiPermission] = cast(Set[PypiPermission], set())
 
-        data = self.client.get_all(op, page_on='organization.repositories')
+                for t_p in t_permissions:
+                    team_perms = team_perms.union(self.convert_permission(t_p))
 
-        org_type = 'Organization'
-        repo_type = 'Repository'
-        team_type = 'Team'
+                repo_perms['teams'][team] = team_perms
 
-        for repo in data.organization.repositories.edges:
-            permissions = dict(users={}, teams={})
-            repositories[repo.node.name] = permissions
-
-            for collab in repo.node.collaborators.edges:
-                username = collab.node.login
-
-                # We say that the direct permissions are those from the repo itself
-                # and those from the Organization
-                repo_and_org_permissions = {
-                    p
-                    for ps in collab.permission_sources
-                    # Map the Github permission onto a pypicloud permission
-                    for p in self.convert_permission(ps.permission)
-                    if ps.source.__typename__ in {org_type, repo_type}
-                }
-
-                permissions[_users_key][username] = repo_and_org_permissions
-
-                # Permissions can come from the Org, the Repo, or a Team
-                # Here we deal with Team permissions
-                for ps in collab.permission_sources:
-                    if ps.source.__typename__ != team_type:
-                        continue  # noqa: WPS220, nesting depth
-
-                    if ps.source.name not in permissions[_teams_key]:
-                        permissions[_teams_key][ps.source.name] = set()  # noqa: WPS220, nesting depth
-
-                    permissions[_teams_key][ps.source.name].update(self.convert_permission(ps.permission))
+            repositories[repo] = repo_perms
 
         return repositories
 
-    @cache_region.cache_on_arguments()
-    def user_data(self, username: Optional[str] = None) -> Union[UserWithGroups, List[User]]:
+    @overload
+    def user_data(self) -> List[User]:  # pragma: no cover
+        ...
+
+    @overload
+    def user_data(self, username: str) -> User:  # pragma: no cover
+        ...
+
+    @cacheable
+    def user_data(self, username: Optional[str] = None) -> Union[User, List[User]]:
         """Get a list of all users or data for a single user.
 
         Each user is a dict with a 'username' str, and 'admin' bool.
@@ -553,11 +534,11 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
             Union[UserWithGroups, List[User]]: The user with groups, or the list of users.
         """
         if username:
-            return dict(username=username, admin=self.is_admin(username), groups=self.user_groups(username))
+            return {'username': username, 'admin': self.is_admin(username), 'groups': self.user_groups(username)}
 
-        return [dict(username=k, admin=v) for k, v in self.users().items()]
+        return [{'username': k, 'admin': v, 'groups': None} for k, v in self.users().items()]
 
-    @cache_region.cache_on_arguments()
+    @cacheable
     def user_groups(self, username: str) -> List[str]:
         """Return the list of groups for a user.
 
@@ -569,7 +550,7 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         return [team for team, user_roles in self.teams().items() if username in dict(user_roles)]
 
-    @cache_region.cache_on_arguments()
+    @cacheable
     def users(self) -> Dict[str, bool]:
         """Return the list of users and their admin status.
 
@@ -578,16 +559,8 @@ class Access(abc.ABC, IAccessBackend):  # pragma: only-covered-in-integration-te
         """
         LOG.info('Getting users')
 
-        op, org = self.organization_operation()
-
-        membership = org.members_with_role.edges
-        membership.role()
-
-        membership.node.login()
-
-        data = self.client.get_all(op, page_on='organization.members_with_role')
-
-        return {m.node.login: (m.role.lower() == GithubRoles.admin.value) for m in data.organization.members_with_role.edges}
+        members = self.client.get_organization_members(self.organization_name)
+        return {user.login: role == OrganizationMemberRole.admin for user, role in members}
 
     def check_health(self) -> Tuple[bool, str]:
         """Check the health of the access backend.
